#!/usr/bin/env python
# -*- coding: utf-8 -*-

# twisted import
from twisted.python import log

# nevow import
from nevow import tags as T
from nevow.i18n import _, PlaceHolder

# bewype ui import
from bewype.core import ui, utils
# bewype db import
from bewype.core.db import Query
# bewype meassge import
from bewype.core.message import RpcMessage, RpcResult

class Component(ui.Frame):
    """Base class for a widgets able to trigger actions and modify the db table
    passed as parameter. These offer a common way to communicate with ajax
    widget clients. Here is the list of currently support actions:

    * add
    * delete
    * check (used in a recaptcha widget for example)
    * edit, cancel, reload
    * update
    * upload

    Corresponding methods should be overriden in the child class for a specific
    mecanism. Additionnal callbacks can be added in the child class also, then
    corresponding ajax client should be present in the static scripts and
    implemented.
    """

    def __init__(self, widget_name=None, orm_cls_name=None, label=None):
        """Creates a new base component for a widgets able to trigger actions
        and modify the db table passed as parameter.

        :param widget_name: common widget name of the component
        :param orm_cls_name: db table on wich the component apply actions
        :param label: label of the widget, not mandatory.
        """
        # Create parent
        ui.Frame.__init__(self)
        # do not manage label as frame title but simple label
        self._label = label
        # set widget name
        self.widget_name = widget_name
        # one query for all
        if orm_cls_name is None:
            self._q = None
        else:
            self._q = Query(orm_cls_name)
        # create unique group key
        self.group_key = utils.generate_key(12)

    def _add_component_templates(self, file_, key_dict=None, dir_path=None):
        # get value key for templating
        _value_key = self.get_value_key()
        if key_dict is None:
            key_dict = {"value_key": _value_key}
        else:
            key_dict["value_key"] = _value_key
        # call parent with updated template
        ui.Widget._add_component_templates(self, file_, key_dict=key_dict,
                dir_path=dir_path)

    def add_static_resources(self, file_=None):
        ui.Widget.add_static_resources(self, file_=file_)

    def get_actions(self):
        """Default []
        """
        return []

    def get_value_key(self):
        """Please implement this!
        """
        raise NotImplementedError("Please implement this!")

    def get_field(self, item):
        """Please implement this!
        """
        raise NotImplementedError("Please implement this!")

    def set_hidden_id(self, item):
        # get value - default ""
        if self.widget_name is None \
        or self._q is None:
            # no hidden key
            return
        elif item is None:
            _value = ""
        # get value according the key name
        else:
            _value = getattr(item, self._q.key_id)
        # get hidden tag
        _hidden = T.input(type_="hidden",
                id_="%sHiddenItemId" % self.id_, value=_value)
        self.add(_hidden)

    def set_widget_element(self, widget_element):
        # call parent
        ui.Widget.set_widget_element(self, widget_element)
        # a little bit of magic - TODO set frame id with widget element id
        if self.widget_name is None:
            pass
        else:
            self.class_ = "%s%s" % (
                    utils.get_class_name(self.widget_name),
                    self.widget_element.id_in_letters())
            # set unique id
            self.id_ = "%s%s%s" % (
                    self.class_,
                    utils.get_class_name(self.get_value_key()),
                    self.__class__.__name__)

    def _realize(self, data, item_id=None):
        # get first element to edit
        if item_id is None:
            _item = None
        # None
        else:
            _item = self._select(item_id)
        # add hidden id
        self.set_hidden_id(_item)
        # add recaptcha item
        self.add(self.get_field(_item))

    def realize_not_protected(self, data):
        self._realize(data)

    def realize_not_logged(self, data):
        self._realize(data)

    def realize_logged(self, avatarId, data):
        self._realize(data)

    def realize_not_authorized(self, avatarId, data):
        self._realize(data)

    def _get_cb_dict(self):
        return {
                "add": self.do_add,
                "cancel": self.do_reload,
                "check": self.do_check,
                "delete": self.do_delete,
                "edit": self.do_reload,
                "reload": self.do_reload,
                "update": self.do_update,
                "upload": self.do_upload,
                "append": self.do_append,
                "remove": self.do_remove
                }

    def _get_action_callback(self, action):
        _action_k = "%s%s" % (self.id_, utils.get_class_name(action))
        return _action_k, [self.widget_element, self._get_cb_dict()[action]]

    def get_callbacks(self):
        _action_list = self.get_actions()
        # add default cb ids
        _action_list.append("cancel")
        _action_list.append("edit")
        _action_list.append("reload")
        _d = dict(map(self._get_action_callback, _action_list))
        return _d

    def _add(self, data, data_dict=None):
        """Can be overriden .. but please return the rpc result dict ;)
        """
        if self._q is None:
            return None
        elif data_dict is None:
            # parser message
            _d_dict = RpcMessage(data).get_dict()
        else:
            pass
        # create new item
        _new_item = self._q.create(**_d_dict)
        # return rpc result            
        return None if _new_item is None \
                else getattr(_new_item, self._q.key_id)

    def _select(self, item_id):
        """Can be overriden .. but please return the rpc result dict ;)
        """
        if self._q is None:
            return None
        else:
            return self._q.get_item(self._q.key_id, item_id)

    def _delete(self, item_id):
        """Can be overriden .. but please return the rpc result dict ;)
        """
        return None if self._q is None \
                else self._q.remove_item(self._q.key_id, item_id)

    def _get_value(self, item):
        # get value key
        _value_key = self.get_value_key()
        # get value
        if item is None \
        or _value_key is None \
        or not hasattr(item, _value_key):
            return None
        else:
            # get item
            return getattr(item, _value_key)

    def _set_value(self, item, value):
        # get value key
        _value_key = self.get_value_key()
        # set value
        if item is None \
        or _value_key is None \
        or not hasattr(item, _value_key):
            return False
        else:
            # set item
            setattr(item, _value_key, value)
            return self._q.commit()

    def do_add(self, data, public=False):
        # get avatar id
        _avatar_id = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        if _avatar_id is None \
        and public is False:
            # not connected result
            _r.error(_("Not connected!"))
            return _r
        else:
            _item_id = self._add(data)
            # do add
            if _item_id is None:
                _r.error(_("Add failed!"))
                return _r
            else:
                _r.success(_("Successfully added!"))
                _r.add("itemId", _item_id)
                return _r

    def do_delete(self, data, public=False):
        # get avatar id
        _avatarId = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        # not connected
        if _avatarId is None \
        and public is False:
            _r.error(_("Not connected!"))
            return _r
        # is connected
        else:
            # get user name
            _item_id = RpcMessage(data).get_int("itemId")
            # parse data
            if _item_id is None:
                # error
                _r.error(_("Invalid delete request!"))
                return _r
            # do remove
            else:
                # get delete result
                _r_delete = self._delete(_item_id)
                # call child delete method
                if _r_delete is True:
                    _r.success(_("Successfully deleted!"))
                elif isinstance(_r_delete, str) \
                or isinstance(_r_delete, PlaceHolder):
                    _r.error(_r_delete)
                else:
                    _r.error(_("Delete failed!"))
                return _r

    def do_update(self, data, public=False):
        # get avatar id
        _avatarId = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        # not connected
        if _avatarId is None \
        and public is False:
            _r.error(_("Not connected!"))
            return _r
        # is connected
        else:
            # get content
            _m = RpcMessage(data)
            # get item
            _item = self._select(_m.get_int("itemId"))
            # check found
            if _item is None:
                # simple message for users
                _r.error(_("Item not found!"))
                # reCaptcha result
                return _r
            else:
                # message value
                _value = _m.get_val(self.get_value_key())
                # do set
                _r_set = self._set_value(_item, _value)
                # check result type: bool or txt
                if _r_set is True:
                    _r.success(_("Item sucessfully updated"))
                elif isinstance(_r_set, str) \
                or isinstance(_r_set, PlaceHolder):
                    _r.error(_r_set)
                else:
                    _r.error(_("Update failed"))
                # return result at the end
                return _r

    def do_reload(self, data, public=False):
        # get avatar id
        _avatarId = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        # not connected
        if _avatarId is None \
        and public is False:
            _r.error(_("Not connected!"))
            return _r
        # is connected
        else:
            # get item
            _item_id = RpcMessage(data).get_int("itemId")
            # always ok
            _r.success(_("OK"))
            _r.add("itemId", _item_id)
            # get value_key
            _value_key = self.get_value_key()
            # little check
            if _value_key is None:
                pass
            else:
                # get item
                _item = self._select(_item_id)
                # get value
                _value = self._get_value(_item)
                # get value
                _r.add(_value_key, _value)
            # result
            return _r

    def do_append(self, data, public=False):
        return RpcResult()

    def do_check(self, data, public=False):
        return RpcResult()

    def do_upload(self, data, public=False):
        return RpcResult()

    def do_remove(self, data, public=False):
        return RpcResult()

