#!/usr/bin/env python
# -*- coding: utf-8 -*-

# twisted import
from twisted.python import log

# bewype import
from bewype.core import ui
# bewype db import
from bewype.core.db import query

class Layout(ui.Frame):
    """Widget used for sub-widgets positionning. Is associated to a
    `component.Page` and a `Layout` db item.

    The `WidgetElement` or `Layout` children db items of the layout will be
    displayed according to the layout type: horizontal or vertical.

    When the parent `Page` call the realize method, the `Layout` call all
    the realize methods of its children.
    """

    def __init__(self, page, layout_item):
        """Creates a new `Layout` instance with the associated parent `Page`
        instance and the corresponding `Layout` db item.

        :param page: parent `Page`
        :param layout_item: `Layout` db item
        """
        #
        ui.Frame.__init__(self)
        # init globals
        self.page = page
        # init layout vars
        if layout_item is None:
            #
            self.id_ = None
            self.class_ = None
            self.layout_type = None
        else:
            #
            self.id_ = "Layout%sFrame" % layout_item.id_in_letters()
            self.class_ = "Layout%s" % layout_item.id_in_letters()
            # get the page layout type
            self.layout_type = layout_item.layout_type
        # _widgets objects of the generic widget container
        self._widgets = list()
        self._children = list()
        # simple children factory
        if layout_item is None:
            pass
        else:
            # add childre widget
            map(self._add_element, layout_item.widget_elements)
            # add children layout
            map(self._add_child, layout_item.children)

    def _add_element(self, widget_element):
        # get the widget class from db element
        _widget_class = widget_element.widget.get_component_cls()
        # init object
        _w = _widget_class()
        # set widget page
        _w.set_page(self.page)
        # set widget element
        _w.set_widget_element(widget_element)
        # init static ressource
        _w.add_static_resources()
        # add it to the generice widget list for rendering
        self._widgets.append(_w)

    def _add_child(self, layout_item):
        # get child widget from layout
        _w = Layout(self.page, layout_item)
        # append to children list
        self._children.append(_w)

    def realize(self, data):
        """Calls all the layout children widgets and layouts realize methods to
        prepare the layout content to be rendered throw the widget
        `get_docFactory` base method.
        """
        #
        self.widgets = list()
        # box factory according to the layout type
        if self.layout_type is None \
        or self.layout_type.layout_type_name == u"type_vertical":
            _box = ui.VBox(class_=self.class_)
        else:
            _box = ui.HBox(class_=self.class_)
        # do realize all the widget children and add them to the layout
        for _w in self._widgets:
            _w.realize(data)
            _box.add(_w)
        # do realize all the layout children and add their boxes to the layout
        for _c in self._children:
            _c.realize(data)
            _box.add(_c)
        # add the realized box to the frame
        self.add(_box)

