#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os
from ConfigParser import SafeConfigParser

# default bewype db repository
import bewype.db.repository

class Config(object):
    """Simple config singleton for a bewype application. Offers shortcut methods
    to obtain config values directly.
    """

    class __Singleton:
        """Our singleton object.
        """

        def __init__(self, config):
            """Create the new singleton with the application config.

            :param config: SafeConfigParser object for all the application
            :see: `ConfigParser.SafeConfigParser`
            """
            # ensure config
            if config is None:
                self.config = SafeConfigParser()
                self.config.read("config.ini")
            else:
                self.config = config

        def get(self, path, type_=str):
            """A little jQuery like shortcut for the `get_value` method.

            :param path: something like "section>option"
            :param type_: type of the expected value. Default: str
            :return: expected value in the expected type or None
            :rtype: `object`
            """
            # get value
            _value = self.get_value(*path.split(">"), type_=type_)
            # check and return
            return None if _value is None or _value == "" else _value

        def get_value(self, section, option, type_=str):
            """Simple config value getter to avoid exception risk when getting
            a config value. If the section and option exist, returns the
            corresponding value, None otherwise.

            The `type_` parameter specify the expected option type and
            return.

            :param section: section name of the expected value
            :param option: option name name of the expected value
            :param type_: type of the expected value. Default: str
            :return: expected value in the expected type or None
            :rtype: `object`
            """
            # check has config
            if self.config is None:
                return None
            # check value exist
            elif self.config.has_section(section) \
            and self.config.has_option(section, option):
                # check expected value type
                if type_ is int:
                    return self.config.getint(section, option)
                elif type_ is bool:
                    return self.config.getboolean(section, option)
                elif type_ is str:
                    return self.config.get(section, option)
                # unexpected type ??
                else:
                    return None
            # value does not exist                
            else:
                # do nothing
                return None

        def get_section(self, section):
            """Returns a dict object with the content of the section passed as
            parameter. None if the section does not exist.

            :param section: section name of the expected value
            """
            # check has config
            if self.config is None:
                return None
            # check section exist
            elif self.config.has_section(section):
                # build and return dict with the given section
                return dict(self.config.items(section))
            # not found
            else:
                # do nothing
                return None

        def deliver_static_files(self):
            """Ensure default deliver static files flag: False.
            """
            _flag = Config().get("app>deliver_static_files", type_=bool)
            return not _flag is False

        def date_format(self, default=False):
            """Ensure default date format: "%Y-%m-%d".
            """
            _date_fmt = self.get("app>date_format")
            if default is True \
            or _date_fmt is None:
                return "%Y-%m-%d"
            else:
                return _date_fmt

        def js_date_format(self):
            """Ensure default js date format: "Y-m-d".
            """
            _date_fmt = self.get("app>date_format")
            if _date_fmt is None:
                return "Y-m-d"
            else:
                return _date_fmt.replace("%", "")

        def log_file(self, port_offset=0, secure=False):
            """Ensures default log file value: "bewype.log".
            """
            # get config value
            _log_file = Config().get("logging>file")
            # ensure value
            if _log_file is None:
                _log_file = "bewype-%d" % port_offset
            else:
                _log_file = "%s-%d" % (_log_file, port_offset)
            # add public or secure info and return
            if secure is True:
                return "%s-sec.log" % _log_file
            else:
                return"%s-pub.log" % _log_file

        def log_dir(self):
            """Ensures default log dir value: ".".
            """
            # get config value
            _log_dir = Config().get("logging>dir")
            # # default current dir
            if _log_dir is None:
                return "."
            else:
                return _log_dir

        def log_length(self):
            """Ensures default log length value: 10000000.
            """
            # get config value
            _log_length = Config().get("logging>length", type_=int)
            # default 10000000
            if _log_length is None:
                return 10000000
            else:
                return _log_length

        def resource_path(self):
            """Ensure default resource path: "./resource".
            """
            # get resource path
            _path = Config().get("path>resource")
            # check and return
            return "resource" if _path is None else _path

        def static_path(self):
            """Ensure default static path: "./static".
            """
            # get resource path
            _path = Config().get("path>static")
            # check and return
            return "static" if _path is None else _path

        def db_url(self):
            """Ensure default db url: "sqlite:///bewype_www.db".
            """
            # get value
            _db_url = Config().get("db>url")
            # ok or default
            return "sqlite:///bewype_www.db" if _db_url is None else _db_url

        def db_repository(self):
            """Ensure default db repository: "bewype/db/repository".
            """
            # get main repository path
            _repository = Config().get("db>repository")
            # check
            if _repository is None:
                return os.path.dirname(bewype.db.repository.__file__)
            else:
                return _repository

    # singleton instance
    instance = None

    def __new__(c, config=None, force=False):
        """Singleton new init.
        """
        # if doesn't already initialized
        if not Config.instance \
        or force is True:
            # create a new instance
            Config.instance = Config.__Singleton(config)
        # return the manager object
        return Config.instance

