#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from functools import partial

# PyYaml import
import yaml

class YamlBaseLoader(object):
    """Simple config like oriented parser based on Yaml file format.

    Here is the config file expected structure::

        config_key:
            section_key:
                value_key: value_content
                value_key: ...
            section_key:
                ...
        config_key:
            ...

    Why a config_key?

    It permits to set several config for the same object. Then it can choose
    the good one according a specific context.
    """

    def __init__(self, file_obj=None):
        """Create the loader object an serialize the file content of the given
        yaml file path.

        :param file_obj: file object with yaml content.
        """
        # get serialize content
        self.serialized_content = None
        # serialize file and init content
        if file_obj is None:
            pass
        else:
            self.serialize(file_obj)

    def serialize(self, file_obj):
        """Serialize the file content of the given yaml file path using PyYaml
        library.

        Here is yaml config file structure sample::

            >>> from tempfile import TemporaryFile
            >>> _f = TemporaryFile()
            >>> _f.write(''' 
            ... config_key:
            ...     section_key_1:
            ...         value_key_11: value_content_11
            ...     section_key_2:
            ...         value_key_21: value_content_21
            ... ''')
            >>> _f.seek(0)

        Create a yaml base loader object and serialize our content::

            >>> _b_loader = YamlBaseLoader()
            >>> _b_loader.serialize(_f)

        Then serialized content should be a simple dict at the end::

            >>> _b_loader.serialized_content == {
            ... "config_key": {
            ...     "section_key_1": {
            ...         "value_key_11": "value_content_11"
            ...         },
            ...     "section_key_2": {
            ...         "value_key_21": "value_content_21"
            ...         }
            ...     }
            ... }
            True

        :param file_obj: file object with yaml content.
        """
        # read yaml content
        _yml_content = file_obj.read()
        # do serialize content
        self.serialized_content = yaml.load(_yml_content)

    def load(self, config_keys=None):
        """Start the content loading process. Mapping the config keys first, the
        loading order can be specify using the config_keys parameter.

        Here is yaml config file structure sample::

            >>> from tempfile import TemporaryFile
            >>> _f = TemporaryFile()
            >>> _f.write('''
            ... 1000:
            ...     100:
            ...         10: 1
            ...         20: 2
            ...     200:
            ...         10: 1
            ... ''')
            >>> _f.seek(0)

        Create a yaml base loader object and serialize our content::

            >>> _b_loader = YamlBaseLoader(file_obj=_f)

        We override the load_value method for the data loading test::

            >>> _b_loader.__result = 0
            >>> def _our_load_value_func(c_key, s_key, v_key):
            ...     _v = _b_loader.get_value_content(c_key, s_key, v_key)
            ...     _b_loader.__result += c_key + s_key + v_key + _v

            >>> _b_loader.load_value = _our_load_value_func

        Here is what load do::

            >>> _b_loader.load()
            >>> _b_loader.__result == 3444
            True

        :param config_keys: ordered list of the config keys to be loaded.
        """
        # here we loop manually for loading order reason
        # or sot keys it can matter
        if config_keys is None:
            # get dict keys
            config_keys = self.serialized_content.keys()
            # sort key
            config_keys.sort()
        else:
            pass
        # load first yaml level
        map(self.load_config, config_keys)

    def get_config_content(self, config_key):
        """Simple getter to obtain a config content of the config key passed as
        parameter.

        :param config_key: key of the expected config content.
        :return: the dictionary found in the global serialized content under the
                 given config key.
        :rtype: `dict`
        """
        # get config content
        return self.serialized_content[config_key]

    def load_config(self, config_key):
        """Load a config content of a given config key. Sort the section keys
        and map a config content with the keys using the load section method.

        :param config_key: key of the expected config content to load.
        """
        # get content
        _config_content = self.get_config_content(config_key)
        # little check
        if _config_content is None:
            pass
        else:
            # get 2cd level keys
            _section_keys = _config_content.keys()
            # sort keys
            _section_keys.sort()
            # load 3rd level content
            map(partial(self.load_section, config_key), _section_keys)

    def get_section_content(self, config_key, section_key):
        """Simple getter for section content of the config key and section passed
        as parameter.

        :param config_key: key of the expected config content.
        :param section_key: key of the expected section content.
        :return: the dictionary found in the global serialized content under the
                 given config and section keys.
        :rtype: `dict`
        """
        # get config content
        _config_content = self.get_config_content(config_key)
        # get section content
        return _config_content[section_key]

    def load_section(self, config_key, section_key):
        """Load a section content of a given config and section keys. Sort the
        value keys and map a section content with the keys using the load value
        method.

        :param config_key: key of the expected config content to load.
        :param section_key: key of the expected section content to load.
        """
        # get section content
        _section_content = self.get_section_content(config_key, section_key)
        # little check
        if _section_content is None:
            pass
        else:
            # get 3rd level keys
            _value_keys = _section_content.keys()
            # sort keys
            _value_keys.sort()
            # load final level content
            map(partial(self.load_value,config_key, section_key), _value_keys)

    def get_value_content(self, config_key, section_key, value_key):
        """Simple getter for value content of the config key and section passed
        as parameter.

        :param config_key: key of the expected config content.
        :param section_key: key of the expected section content.
        :param value_key: key of the expected value content.
        :return: the dictionary found in the global serialized content under the
                 given config, section and value keys.
        :rtype: `dict`
        """
        # get section content
        _section_content = self.get_section_content(config_key, section_key)
        # get value content
        return _section_content[value_key]

    def load_value(config_key, section_key, value_key):
        """Load a value content of a given config and section keys. Here we have
        all the keys to load a value in a database for example. This method
        should be overriden to proceed to a specific value loading.

        :param config_key: key of the expected config content to load.
        :param section_key: key of the expected section content to load.
        :param value_key: key of the expected value content to load.
        """
        raise NotImplementedError("Please implement this!")

