#!/usr/bin/env python
# -*- coding: utf-8 -*-

# bewype data import
from bewype.core.data.yaml_base_loader import YamlBaseLoader

# bewype db import
from bewype.core.db import Query

class YamlConfigLoader(YamlBaseLoader):
    """Load bewype widget element config values from a yaml config file. Nice
    way to setup default widget config for a specific language.

    Here is the expected yaml config file structure::

        config_name:
            config_section:
                config_key:
                    value_type: bool
                    value_text: True

    :see: `YamlBaseLoader`
    """

    def __init__(self, widget_element_id, language_id, file_obj=None):
        """Create the loader object for a specific widget element id and
        language id. The file object passed as parameter automatically trigger
        the data serialization.

        :param widget_element_id: db item id of the config widget element.
        :param language_id: db item id of the config language.
        :param file_obj: file object with yaml content for the config to be
                         loaded.
        :see: `YamlBaseLoader`
        """
        # init the yaml base loader parent
        YamlBaseLoader.__init__(self, file_obj=file_obj)
        # set globals
        self.widget_element_id = widget_element_id
        self.language_id = language_id

    def _exist(self, config_key, section_key, value_key):
        """Checks if config db item already exist in database.

        :param config_key: used as config name for the config db item.
        :param section_key: used as section key for the config db item.
        :param value_key: used for value key and value for the config db item.

        :return: the boolean from the Query.exist method result.
        :rtype: `boolean`

        :see: `Query`
        """
        # get config query
        _q = Query("Config")
        # prepare exist criteria
        _crit_list = [
                _q.obj.config_name==config_key,
                _q.obj.config_section==section_key,
                _q.obj.config_key==value_key
                ]
        # return exist result
        return _q.exist(_crit_list)

    def load_value(self, config_key, section_key, value_key):
        """Create a config db item with the the yaml value. Then the new
        item will be added to the current db session and commit.

        :param config_key: used as config name for the config db item.
        :param section_key: used as section key for the config db item.
        :param value_key: used for value key and value for the config db item.

        :see: `Config`
        """
        # get value content
        _value_content = \
                self.get_value_content(config_key, section_key, value_key)
        # little check
        if _value_content is None:
            pass
        # check if value already exist
        elif self._exist(config_key, section_key, value_key):
            pass
        # check value content keys
        elif _value_content.has_key("value_type") \
        and _value_content.has_key("value_text"):
            # get values
            _value_type = _value_content["value_type"]
            _value_text = _value_content["value_text"]
            # get ids
            _language_id = self.widget_element_id
            _widget_element_id = self.language_id
            # get config query
            _q = Query("Config")
            # prepare dict of value to insert
            _insert_dict = {
                    "config_name": config_key,
                    "config_section": section_key,
                    "config_key": value_key,
                    "value_type": _value_type,
                    "value_text": _value_text,
                    "language_id": _language_id,
                    "widget_element_id": _widget_element_id
                    }
            # Create config item
            _config_item = _q.create(**_insert_dict)
        # nothing to do
        else:
            pass

