#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import gettext, os, subprocess

# babel import
from babel.messages import catalog, pofile

# nevow import
from twisted.python import log

# bewype config import
from bewype.core.config import Config
# bewype resource import
from bewype.core.resource import Path, Resource

class I18nUpdater(object):
    """This class aim to update an i18n .mo file for a specific language of a
    bewype application. All bewype modules should have its own i18n .po file.

    By default all i18n files are present in the <resource_path>/i18n folder.
    The <resource_path> folder come from the config.

    The updater merger all .po files in one then compile a new .mo file for the
    given language. At the end the python i18n cache is cleared to reload the
    new language values.
    """

    def __init__(self, language_code):
        """Creates a new updater object. Set the i18n dir and language code of
        the .mo file to update.

        :param language_code: language code of the mo file to update, ex.: en,
                              fr, Etc.
        """
        # set i18n language code to manage
        self.language_code = language_code
        # set the common i18n path for the current language
        self.i18n_path = self.get_i18n_path()

    def get_i18n_path(self):
        """Returns the .mo dir path for the current language and the resource
        path from the application config.
        """
        # set resource path from config
        _resource_path = Config().get("path>resource")
        # little check
        if self.language_code is None \
        or _resource_path is None:
            return None
        else:
            # update path
            return os.path.join(
                    _resource_path, "i18n", self.language_code, "LC_MESSAGES")

    def write_po_file(self, catalog):
        """Writes the new main .po file in the current i18n dir from a given
        catalog.

        :param catalog: representation of a message catalog
        :see: `catalog`
        """
        # little check
        if self.i18n_path is None:
            return
        else:
            # dest file path
            _dest_file_path = os.path.join(self.i18n_path, "bewype.po")
        # open file
        _dest_f = open(_dest_file_path, "wb")
        _dest_f.write(r"""
# French translations for PROJECT.
# Copyright (C) 2010 ORGANIZATION
# This file is distributed under the same license as the PROJECT project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2010.
#
#, fuzzy
msgid ""
msgstr ""
"#-#-#-#-#  bewype.po (PROJECT VERSION)  #-#-#-#-#\n"
"Project-Id-Version: PROJECT VERSION\n"
"Report-Msgid-Bugs-To: EMAIL@ADDRESS\n"
"POT-Creation-Date: 2010-04-05 15:22+0200\n"
"PO-Revision-Date: 2010-04-05 15:22+0200\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: %s <LL@li.org>\n"
"Plural-Forms: nplurals=2; plural=(n > 1)\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel 0.9.4\n"
"#-#-#-#-#  bewype.po (PROJECT VERSION)  #-#-#-#-#\n"
""" % self.language_code)
        # new po file
        pofile.write_po(_dest_f, catalog, omit_header=True)
        # close dest file
        _dest_f.close()

    def merge_po_files(self):
        """Merges all the .po files of the current i18n dir in a main file named
        bewype.po. First it builds and update a babel `Catalog` object,
        then call the `write_po_file` method to write the .po file.

        :see: `catalog`
        """
        # little check
        if self.i18n_path is None \
        or not os.path.exists(self.i18n_path):
            return
        else:
            # init lines to write list
            _dest_catalog = catalog.Catalog(locale=self.language_code)
        # merge to file
        for _po_file_name in os.listdir(self.i18n_path):
            # get file ext
            _f_ext = Path(_po_file_name).get_ext()
            # check file type
            if _f_ext == "po":
                # get current file path
                _po_file_path = os.path.join(self.i18n_path, _po_file_name)
                # open
                _f = open(_po_file_path)
                # get babel catalog
                _cat = pofile.read_po(_f, locale=self.language_code,
                        domain="bewype")
                # close file
                _f.close()
                # update current catalog
                for _m in _cat:
                    _dest_catalog[_m.id] = _m
            else:
                continue
        # do write
        self.write_po_file(_dest_catalog)

    def update_mo_file(self, do_merge=True):
        """Triggers the .mo file update. First removes the old file and merges
        all .po files in a new main .po source file. Then calls the system
        `msgfmt` command to recreate the new .mo file.

        At the end we clear the `_translations` dict of the python gettext
        module to force the .mo file reloading.
        """
        # little check
        if self.i18n_path is None:
            return
        else:
            # get main mo file path
            _mo_res_path = os.path.join(self.i18n_path, "bewype.mo")
            _po_res_path = os.path.join(self.i18n_path, "bewype.po")
            # merge .po files or re-compile only?
            if do_merge is True:
                # remove old po file
                Resource(_po_res_path).remove()
                # remove old mo file
                Resource(_mo_res_path).remove()
                # merge all po files in one
                self.merge_po_files()
            # just re-compile current .po file
            else:
                # remove old mo file
                Resource(_mo_res_path).remove()
            # compile po file
            try:
                # call args
                _c_args = ["msgfmt", "-o", _mo_res_path, _po_res_path]
                # do os call in sub process
                subprocess.check_call(_c_args)
                # clear python i18n cache
                gettext._translations = {}
            # Oops
            except subprocess.CalledProcessError, e:
                # ERROR
                log.err("[resource] regen_mo_file - e: %s" % e)

