#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os

# twisted import
from twisted.python import log

# bewype config import
from bewype.core.config import Config

class Path(object):
    """This class gives additional information and corresponding resource path,
    server path, static path for a given path.
    """

    def __init__(self, path=None, py_file=None, language_code=None, widget_element=None):
        """Creates the manager instance with the path to manage.

        :param path: a resource path
        """
        # set the main path variable
        self.path = path
        self.py_file = py_file
        self.language_code = language_code
        self.widget_element = widget_element

    def set_path(self, path):
        """
        """
        self.path = path

    def get_path(self):
        """
        """
        return self.path

    def set_widget_element(self, widget_element):
        """
        """
        self.widget_element = widget_element

    def get_path(self):
        """
        """
        return self.widget_element

    def has_valid_path(self):
        """Ensures value is correctly then return True, otherwise False.

        Here is what we valid, very simple::

            >>> Path('_').has_valid_path()
            True

            >>> Path(None).has_valid_path()
            False

            >>> Path('').has_valid_path()
            False

        :return: True if `path` contain text
        """
        return self.path is not None \
                and self.path.strip() != ""

    def get_ext(self):
        """Returns the extension of the current resource file. Works only if
        current `path` correspond to a file.

        Do not check if the resource exists because create one can require
        extension first.

        We can use it in one line for all our resources::

            >>> Path('test.py').get_ext() == 'py'
            True

            >>> Path('test').get_ext() is None
            True

            >>> Path('./test').get_ext() is None
            True

        :return: the resource extension of the current resource path or None
        :rtype: `str`
        """
        if self.has_valid_path():
            # get file ext
            _dir, _file_name = os.path.split(self.path)
            if _file_name.find(".") == -1:
                return None
            else:
                _ext = _file_name.split(".")[-1].strip()
                return _ext.lower()
        else:
            return None

    def get_type(self):
        """Returns the corresponding resource type according to the extension of
        the resource path passed as parameter.

        We can use it in one line for all our resources::

            >>> Path('test.jpg').get_type() == 'img'
            True

            >>> Path('test.py').get_type() is None
            True

        Note: python file are currently not delivered by our application server.
        It's not a missing point but the current bewype specificatin issue ;)

        :return: the resource type for the current resource extension or None
        :rtype: `str`
        """
        # get file extension for test stuff
        _ext = self.get_ext()
        # TODO see if hard coded values are the good way :s!not sure!
        if _ext in ["png", "jpg", "jpeg", "gif"]:
            return "img"
        elif _ext in ["css"]:
            return "css"
        elif _ext in ["po"]:
            return "i18n"
        elif _ext in ["txt", "html", "xml", "mako", "tmpl"]:
            return "tmpl"
        elif _ext in ["js"]:
            return "script"
        else:
            return None

    def get_py_file_dir(self, py_file, sub_path=None):
        """Returns the static dir path nearby a python file object or None if
        the corresponding static path does not exist.

        Here is an explicit exemple of the concept, first without static dir we
        get None if we create a dir folder then we get the current nearby static
        path::

            >>> Path().get_py_file_dir(__file__, sub_path='sub') is None
            True

            >>> _path = os.path.join(os.path.dirname(__file__), 'sub')
            >>> os.mkdir(_path)
            >>> Path().get_py_file_dir(__file__, sub_path='sub') == _path
            True

            >>> os.rmdir(_path)

        :param sub_path: specify a sub_dir to append to the static path,
                         ex.: css, script, img
        :return: the python file nearby static path
        :rtype: `str`
        """
        # get parent path of the py file
        _path = os.path.dirname(py_file)
        # get corresponding static path
        if sub_path is None:
            pass
        else:
            _path = os.path.join(_path, sub_path)
        # path should exist
        if os.path.exists(_path):
            return _path
        # path not found
        else:
            return None

    def get_resource_path_from_py_file(self, py_file, sub_path=None):
        """Returns the resource dir path nearby a python file object or None if
        the corresponding resource path does not exist.

        Should work lilke this::

            >>> Path().get_resource_path_from_py_file(__file__) is None
            True

            >>> _path = os.path.join(os.path.dirname(__file__), 'resource')
            >>> os.mkdir(_path)
            >>> Path().get_resource_path_from_py_file(__file__) == _path
            True

            >>> os.rmdir(_path)

        :param sub_path: specify a sub_dir to append to the resource path,
                         ex.: css, script, img
        :return: the python file nearby static path
        :rtype: `str`
        """
        # get corresponding static path
        if sub_path is None:
            _static_path = "resource"
        else:
            _static_path = os.path.join("resource", sub_path)
        # path with py file parent dir
        return self.get_py_file_dir(py_file, sub_path=_static_path)

    def get_static_path_from_py_file(self, py_file, sub_path=None):
        """Returns the static dir path nearby a python file object or None if
        the corresponding static path does not exist.

        Should work lilke this::

            >>> Path().get_static_path_from_py_file(__file__) is None
            True

            >>> _path = os.path.join(os.path.dirname(__file__), 'static')
            >>> os.mkdir(_path)
            >>> Path().get_static_path_from_py_file(__file__) == _path
            True

            >>> os.rmdir(_path)

        :param sub_path: specify a sub_dir to append to the static path,
                         ex.: css, script, img
        :return: the python file nearby static path
        :rtype: `str`
        """
        # get corresponding static path
        if sub_path is None:
            _static_path = "static"
        else:
            _static_path = os.path.join("static", sub_path)
        # path with py file parent dir
        return self.get_py_file_dir(py_file, sub_path=_static_path)

    def get_tmpl_path_from_py_file(self, py_file, sub_path=None):
        """Returns the tmpl dir path nearby a python file object or None if the
        corresponding tmpl path does not exist.

        Should work lilke this::

            >>> Path().get_tmpl_path_from_py_file(__file__) is None
            True

            >>> _path = os.path.join(os.path.dirname(__file__), 'tmpl')
            >>> os.mkdir(_path)
            >>> Path().get_tmpl_path_from_py_file(__file__) == _path
            True

            >>> os.rmdir(_path)

        :param sub_path: specify a sub_dir to append to the tmpl path,
                         ex.: css, script, img
        :return: the python file nearby tmpl path
        :rtype: `str`
        """
        # get corresponding static path
        if sub_path is None:
            _tmpl_path = "tmpl"
        else:
            _tmpl_path = os.path.join("tmpl", sub_path)
        # path with py file parent dir
        return self.get_py_file_dir(py_file, sub_path=_tmpl_path)

    def build_app_path(self, base_path, use_type=True):
        """
        """
        # little check
        if base_path is None:
            return None
        else:
            pass
        # get element id in letters -> not added if no element set
        if self.widget_element is None:
            pass
        else:
            # get element id
            _e_id = "Element%s" % self.widget_element.id_in_letters()
            # add element id to the path
            base_path = os.path.join(base_path, _e_id)
        # get template type
        _r_type = self.get_type()
        if _r_type is None:
            return None
        elif use_type is True:
            # add type to the path
            base_path = os.path.join(base_path, _r_type)
        else:
            pass
        # return css or script path
        if _r_type == "i18n":
            # update path
            return os.path.join(
                    base_path, self.language_code, "LC_MESSAGES", self.path)
        elif _r_type == "css" \
        or _r_type == "script" \
        or _r_type == "tmpl":
            return os.path.join(base_path, self.path)
        # require page for other paths
        elif _r_type == "img":
            if self.language_code is None \
            or self.widget_element is None:
                return os.path.join(base_path, self.path)
            else:
                return os.path.join(base_path, self.language_code, self.path)
        else:
            return None

    def get_tmpl_dir(self):
        """
        """
        if self.py_file is None:
            return None
        else:
            return self.get_tmpl_path_from_py_file(self.py_file)

    def get_tmpl_path(self, use_type=True):
        """
        """
        # init result resource path
        _tmpl_path = self.get_tmpl_dir()
        # compute path
        return self.build_app_path(_tmpl_path, use_type=use_type)

    def get_resource_dir(self, use_py_file=True):
        """
        """
        if self.py_file is None \
        or use_py_file is False:
            return Config().resource_path()
        else:
            return self.get_resource_path_from_py_file(self.py_file)

    def get_resource_path(self, use_py_file=True, use_type=True):
        """
        """
        # init result resource path
        _resource_path = self.get_resource_dir(use_py_file=use_py_file)
        # compute path
        return self.build_app_path(_resource_path, use_type=use_type)

    def get_static_dir(self, use_py_file=True):
        """
        """
        if self.py_file is None \
        or use_py_file is False:
            return Config().static_path()
        else:
            return self.get_static_path_from_py_file(self.py_file)

    def get_static_path(self, use_py_file=True, use_type=True):
        """
        """
        # init result static path
        _static_path = self.get_static_dir(use_py_file=use_py_file)
        # compute path
        return self.build_app_path(_static_path, use_type=use_type)

    def get_web_path(self, use_type=True):
        """
        """
        # init result web path
        _web_path = "/static"
        # compute path
        return self.build_app_path(_web_path, use_type=use_type)

