#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import unittest

# bewype db import
from bewype.core.db import Query

# bewype tests import
from bewype.core.tests._db_manager import _DbManager
# init specific db manager singleton for tests
_DbManager()

class TestDBQuery(unittest.TestCase):

    def setUp(self):
        # remove all dummy items for each test
        Query("Dummy").delete_all()

    def tearDown(self):
        # remove all dummy items after each test
        Query("Dummy").delete_all()

    def test_get_item(self):
        # create a new query for the dummy table
        _q = Query("Dummy")
        # set insert dict
        _insert_dict = {"dummy_name": u"test_get_item"}
        # create a new dummy object
        _new_item = _q.create(**_insert_dict)

        # GET EXISTING ITEM

        # get an existing item
        _r = _q.get_item("dummy_name", u"test_get_item")
        # check item value
        assert _r is _new_item, "Found: %s" % _r

        # GET NON-EXISTING ITEM

        # get a non-existing item
        _r = _q.get_item("dummy_name", u"_")
        # check item value
        assert _r is None, "Found: %s" % _r

    def test_get_items(self):
        # create a new query item
        _q = Query("Dummy")
        # add some rows
        # d1
        _d1 = {
                "dummy_name": u"test_get_items_1",
                "dummy_str": u"test_get_items_str"
                }
        _i_1 = _q.create(**_d1)
        # d2
        _d2 = {
                "dummy_name": u"test_get_items_2",
                "dummy_str": u"test_get_items_str"
                }
        _i_2 = _q.create(**_d2)
        # d3
        _d3 = {
                "dummy_name": u"test_get_items_3",
                "dummy_str": u"_"
                }
        _i_3 = _q.create(**_d3)

        # GET SEVERAL ITEMS

        # prepare criteria list
        _crit_list = [_q.obj.dummy_str==u"test_get_items_str"]
        # call get_items method
        _r_list = _q.get_items(_crit_list)

        # check result length
        assert len(_r_list) == 2, "Found: %s" % len(_r_list)
        # check result content
        assert _r_list[0].dummy_str == u"test_get_items_str", \
                "Found: %s" % _r_list[0].dummy_str

        # GET SINGLE ITEM                

        # prepare criteria list
        _crit_list = [_q.obj.dummy_str==u"_"]
        # call get_items method
        _r_list = _q.get_items(_crit_list)

        # check result length
        assert len(_r_list) == 1, "Found: %s" % len(_r_list)
        # check result content
        assert _r_list[0].dummy_str == u"_", "Found: %s" % _r_list[0].dummy_str

        # GET NON-EXISTING ITEM

        # prepare criteria list
        _crit_list = [_q.obj.dummy_str==u""]
        # call get_items method
        _r_list = _q.get_items(_crit_list)

        # check result length
        assert len(_r_list) == 0, "Found: %s" % len(_r_list)

    def test_delete(self):
        # create a new query item
        _q = Query("Dummy")
        # add some rows
        # d1
        _d1 = { "dummy_name": u"test_delete_1" }
        _item = _q.create(**_d1)
        # d2
        _d2 = { "dummy_name": u"test_delete_2" }
        _item = _q.create(**_d2)
        # d3
        _d3 = { "dummy_name": u"test_delete_3" }
        _item = _q.create(**_d3)

        # REMOVE FIRST ITEM

        # prepare criteria list
        _crit_list = [_q.obj.dummy_name==u"test_delete_1"]
        # call delete method
        _q.delete(_crit_list)

        # check number of remaining items
        _r_list = _q.get_all()
        assert len(_r_list) == 2, "Found: %s" % len(_r_list)
        # check item removal
        _r = _q.get_item("dummy_name", u"test_delete_1")
        assert _r is None, "Found: %s" % _r

        # REMOVE SECOND ITEM

        # prepare criteria list
        _crit_list = [_q.obj.dummy_name==u"test_delete_2"]
        # call delete method
        _q.delete(_crit_list)

        # check number of remaining items
        _r_list = _q.get_all()
        assert len(_r_list) == 1, "Found: %s" % len(_r_list)
        # check item removal
        _r = _q.get_item("dummy_name", u"test_delete_2")
        assert _r is None, "Found: %s" % _r

        # REMOVE THIRD ITEM

        # prepare criteria list
        _crit_list = [_q.obj.dummy_name==u"test_delete_3"]
        # call delete method
        _q.delete(_crit_list)

        # check number of remaining items
        _r_list = _q.get_all()
        assert len(_r_list) == 0, "Found: %s" % len(_r_list)
        # check item removal
        _r = _q.get_item("dummy_name", u"test_delete_3")
        assert _r is None, "Found: %s" % _r

        # REMOVE NON-EXISTING ITEM

        # prepare criteria list
        _crit_list = [_q.obj.dummy_name==u"_"]
        # call delete method
        _q.delete(_crit_list)

        # check number of remaining items
        _r_list = _q.get_all()
        assert len(_r_list) == 0, "Found: %s" % len(_r_list)

    def test_exist(self):
        # create a new query item
        _q = Query("Dummy")
        # add a single row
        _d = { "dummy_name": u"test_exist" }
        _item = _q.create(**_d)

        # CHECK EXISTING ITEM

        # prepare criteria list
        _crit_list = [_q.obj.dummy_name==u"test_exist"]
        # get exist flag
        _f = _q.exist(_crit_list)

        # check flag validity
        assert _f is True

        # CHECK NON-EXISTING ITEM

        # prepare criteria list
        _crit_list = [_q.obj.dummy_name==u"_"]
        # get exist flag
        _f = _q.exist(_crit_list)

        # check flag validity
        assert _f is False

    def test_insert(self):
        # create a new query item
        _q = Query("Dummy")

        # FIRST ITEM INSERT

        # prepare insert dict
        _i_dict = {
                "dummy_name": u"test_insert_1",
                "dummy_str": u"test_insert_str",
                }
        # insert first item
        _q.insert(_i_dict)

        # SECOND ITEM INSERT

        # prepare insert dict
        _i_dict = {
                "dummy_name": u"test_insert_2",
                "dummy_str": u"test_insert_str",
                }
        # insert first item
        _q.insert(_i_dict)

        # GET ALL TEST
        _r_list = _q.get_all()
        # check
        assert len(_r_list) == 2, "Found: %s" % len(_r_list)

        # GET ITEMS TEST
        # prepare criteria list
        _crit_list = [_q.obj.dummy_str==u"test_insert_str"]
        # call get_items method
        _r_list = _q.get_items(_crit_list)
        # check
        assert len(_r_list) == 2, "Found: %s" % len(_r_list)

        # GET ITEM TEST
        # first item
        _i_1 = _q.get_item("dummy_name", u"test_insert_1")
        assert _i_1 is not None
        # second item
        _i_2 = _q.get_item("dummy_name", u"test_insert_2")
        assert _i_2 is not None

    def test_select(self):
        # create a new query item
        _q = Query("Dummy")
        # add some rows
        # d1
        _d1 = {
                "dummy_name": u"test_select_3",
                "dummy_str": u"_"
                }
        _item = _q.create(**_d1)
        # d2
        _d2 = {
                "dummy_name": u"test_select_2",
                "dummy_str": u"select_test_str"
                }
        _item = _q.create(**_d2)
        # d3
        _d3 = {
                "dummy_name": u"test_select_1",
                "dummy_str": u"select_test_str"
                }
        _item = _q.create(**_d3)

        # SIMPLE SELECT CHECK

        # prepare criteria list
        _crit_list = [_q.obj.dummy_str==u"select_test_str"]
        # select rows
        _r_list = _q.select(criteria_list=_crit_list)
        # check length
        assert len(_r_list) == 2, "Found: %s" % _r_list
        # first line length check
        assert len(_r_list[0]) == 9, "Found: %s" % _r_list[0]
        # first line content check
        assert _r_list[0][1] == u"test_select_2", "Found: %s" % _r_list[0][1]

        # SELECT WITH COLUMN LIST CHECK

        # prepare criteria list
        _crit_list = [_q.obj.dummy_str==u"select_test_str"]
        # prepare column list
        _col_list = [_q.obj.dummy_str, _q.obj.dummy_name]
        # select rows
        _r_list = _q.select(criteria_list=_crit_list, column_list=_col_list)
        # check length
        assert len(_r_list) == 2, "Found: %s" % len(_r_list)
        # first line length check
        assert len(_r_list[0]) == 2, "Found: %s" % len(_r_list[0])
        # first line content check
        assert _r_list[0][0] == u"select_test_str", "Found: %s" % _r_list[0][1]

        # SELECT WITH ORDER LIST CHECK

        # prepare criteria list
        _crit_list = [_q.obj.dummy_str==u"select_test_str"]
        # prepare column list
        _col_list = [_q.obj.dummy_str, _q.obj.dummy_name]
        # prepare column list
        _ord_by_list = [_q.obj.dummy_name, _q.obj.dummy_str]
        # select rows
        _r_list = _q.select(criteria_list=_crit_list, column_list=_col_list,
                order_by_list=_ord_by_list)
        # check length
        assert len(_r_list) == 2, "Found: %s" % len(_r_list)
        # first line length check
        assert len(_r_list[0]) == 2, "Found: %s" % len(_r_list[0])
        # first line content check
        assert _r_list[0][1] == u"test_select_1", "Found: %s" % _r_list[0][0]

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestDBQuery, "test_db_query"))
    return suite

