#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from functools import partial

# twisted import
from twisted.python import log

# bewype ui import
from bewype.core import utils
# bewype config import
from bewype.core.config import Config
# bewype resource import
from bewype.core.resource import StaticUpdater

def _update_callbacks_dict(dict_to_update, widget):
    # get current dict
    if hasattr(widget, "get_callbacks"):
        # get current cb dict
        _w_callbacks_dict = widget.get_callbacks()
    # quit
    else:
        return
    # pass
    if _w_callbacks_dict is None:
        pass
    else:
        # do update
        for _key in _w_callbacks_dict.keys():
            #
            cb_ = _w_callbacks_dict[_key]
            #
            if dict_to_update.has_key(_key):
                pass
            else:
                dict_to_update[_key] = cb_

class Widget(object):
    """Base object for all the `GTK-like` based html items. Offers base values
    and methods to be integrated in the `Nevow` framework:

        - *get_docFactory* should return encapsulated `tags` to be flattened
        - *realize* prepare the content according to the session status: not \
        protected, not logged, logged, realize not authorized.
        - *add_static_resources* add the resource close to the wiget using its \
        __file__ attribute.
        - *get_callbacks* returns the available ajax callbacks of a widget.
    """

    def __init__(self, id_=None, class_=None):
        """Creates a new `Widget` instance with the common `id_` and `class_`
        parameters.

        :param id_: html identifier of the widget
        :param class_: html class name of the widget
        """
        # main params
        self.id_ = id_
        self.class_ = class_
        #
        self.page = None
        self.widget_name = None
        self.widget_element = None
        #
        self.widgets = list()
        # init layout children list var
        self.callbacks = dict()
        #
        # create unique group key
        self.group_id = utils.generate_key(12)

    def _add_component_resources(self, file_):
        #
        if file_ is None:
            return
        else:
            # init static resource udpater
            _u = StaticUpdater(self.page, py_file=file_)
            # append static dir first
            _u.append_static_dir()

    def _get_layout_ids(self, layout):
        _ids = map(lambda _w: "\"%s\"" % _w.id_in_letters(),
                layout.widget_elements)
        _ids.extend(map(self._get_layout_ids, layout.children))
        return _ids

    def _add_component_templates(self, file_, key_dict=None, dir_path=None):
        if file_ is None \
        or dir_path is None \
        or self.widget_element is None:
            return
        else:
            # init static resource udpater
            _u = StaticUpdater(self.page, py_file=file_,
                    widget_element=self.widget_element)
            # get element id
            _l_ids = self._get_layout_ids(self.widget_element.layout)
            _e_id = self.widget_element.id_in_letters()
            # get default js date format
            _d_fmt = Config().js_date_format()
            # update key dict values for templating
            if key_dict is None:
                key_dict = {
                        "element_id": _e_id,
                        "class_name": self.class_,
                        "widget_id": self.id_,
                        "group_id": self.group_id,
                        "date_format": _d_fmt,
                        }
            else:
                key_dict["element_id"] = _e_id
                key_dict["class_name"] = self.class_
                key_dict["widget_id"] = self.id_
                key_dict["group_id"] = self.group_id
                key_dict["date_format"] = _d_fmt
            # append templated resources
            _u.append_tmpl_dir(key_dict, dir_path=dir_path, widget_id=self.id_)

    def _add_item_resources(self, resource_item):
        # init static resource udpater
        _u = StaticUpdater(self.page, widget_element=self.widget_element)
        _u.append_static_resource(resource_item.file_name)

    def _add_element_resources(self):
        if self.widget_element is None:
            return
        else:
            map(self._add_item_resources, self.widget_element.resources)

    def add_static_resources(self, file_=None):
        """
        """
        # update children resources
        for _w in self.widgets:
            if hasattr(_w, "add_static_resources"):
                _w.add_static_resources()
            else:
                pass
        # add current widget component resources
        self._add_component_resources(file_)
        self._add_component_templates(file_, dir_path="css")
        self._add_component_templates(file_, dir_path="script")
        # add db item set resources
        self._add_element_resources()

    def realize_not_protected(self, data):
        """Realizes not protected content. Call on each request when the
        application is started using the `public_server` config section.

        :param data: args of the http request formatted as a `dict`
        """
        pass

    def realize_not_logged(self, data):
        """Realizes not logged content. Call when user is not already
        authenticated.

        :param data: args of the http request formatted as a `dict`
        """
        pass

    def realize_logged(self, avatarId, data):
        """Realizes logged content. Call when user is authenticated
        and has the required permissions for the widget.

        :param data: args of the http request formatted as a `dict`
        :param avatarId: `user_name` value of the authenticated user
        """
        pass

    def realize_not_authorized(self, avatarId, data):
        """Realizes not authorized content. Call when user is authenticated
        and has not the required permissions for the widget.

        :param data: args of the http request formatted as a `dict`
        :param avatarId: `user_name` value of the authenticated user
        """
        pass

    def realize(self, data):
        """Realize factory for all widgets. Call one of the specific realize
        method according to the session status:

        * realize_not_protected
        * realize_not_logged
        * realize_logged
        * realize_not_authorized

        :param data: args of the http request formatted as a `dict`
        """
        # ensure data
        if data is None:
            data = dict()
        else:
            pass
        # empty widget list
        self.widgets = list()
        # realize factory
        if self.page is not None \
        and self.page.is_protected():
            # get the current avatarId
            _avatarId = self.page.get_avatar_id()
            if _avatarId is None:
                self.realize_not_logged(data)
            else:
                # get user db item
                _user_item = Query("User").get_item("user_name", _avatar_id)
                # check user widget permission
                _u_has_permissions = \
                        _user_item.has_widget_permission(self.widget_element)
                # permission ok
                if _u_has_permissions:
                    self.realize_logged(_avatarId, data)
                # not logged
                else:
                    self.realize_not_authorized(_avatarId, data)
        # not protected by default
        else:
            self.realize_not_protected(data)

    def get_docFactory(self):
        """Returns a `nevow` doc factory content to be flattened. This content
        should prepared when realize the widget. Then the corresponding doc
        factory should generated by the child object using this method.

        This method is specific to each widget and always should be overriden.

        :return: a `nevow` encapsulated tags content, ex.: T.div[T.a["Hello"]]
        :rtype: `nevow.stan.Tag`
        """
        raise NotImplementedError("Please implement this!")

    def get_callbacks(self):
        """Returns the widget callbacks to a top widget or to the current page.
        These callbacks are used for ajax calls. Each `Widget` can be associated
        to a specific client throw the available callbacks.

        The returned `dict` will be used to dispatch ajax calls in the current
        page.

        To specify the available ajax callbacks, this method should overriden by
        the widget child. If it does not purpose ajax support it must return
        an empty dictionnary by default.

        :return: dict of available callbacks for the widget,
                 ex: {callback_id: [widget_element, method]}
        :rtype: `dict`
        """
        return self.callbacks

    def _update_callbacks(self):
        # update children
        for _w in self.widgets:
            if hasattr(_w, "_update_callbacks"):
                _w._update_callbacks()
            else:
                pass
        # prepare update
        _update_children_func = partial(
                _update_callbacks_dict,
                self.page.children_callbacks
                )
        # do update
        map(_update_children_func, self.widgets)

    def set_widget_name(self, widget_name):
        self.widget_name = widget_name

    def get_widget_name(self):
        return self.widget_name

    def set_group_id(self, group_id):
        self.group_id = group_id

    def get_group_id(self):
        return self.group_id

    def set_widget_element(self, widget_element):
        self.widget_element = widget_element

    def get_widget_element(self):
        return self.widget_element

    def set_page(self, page):
        if page is None:
            pass
        else:
            # update children page
            for _w in self.widgets:
                if hasattr(_w, "set_page"):
                    _w.set_page(page)
                else:
                    pass
            # set current page
            self.page = page

    def get_page(self):
        return self.page

