#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os, shutil, subprocess
from functools import partial

# sqlalchemy import
from sqlalchemy.orm import clear_mappers

# migrate import
from migrate.versioning.shell import main as migrate_main

# twisted import
from twisted.application import service
from twisted.python import log, logfile

# bewype component import
from bewype.core.component import Page
# bewype config import
from bewype.core.config import Config
# bewype data import
from bewype.core.data import YamlDbLoader
# bewype db import
from bewype.core.db import model, Query, SessionManager
# bewype resource import
from bewype.core.resource import I18nUpdater
# bewype web import
from bewype.core.web import application, SSLContext

def _init_logging(port_offset, secure):
    # get the log file name
    _log_file = Config().log_file(port_offset=port_offset, secure=secure)
    #
    _log_dir = Config().log_dir()
    _log_length = Config().log_length()
    _f_rotating = logfile.LogFile(_log_file, _log_dir, rotateLength=_log_length)
    # setup logging to use our new logfile
    log.startLogging(_f_rotating)

def _check_db():
    # get session
    _session_manager = SessionManager()
    # check has database otherwise instance could not be created
    if _session_manager is None:
        # get dbuser
        _db_user = Config().get("db>user")
        _db_name = Config().get("db>db_name")
        # call args
        _c_args = ["createdb", "-O", _db_user, "-U", _db_user, _db_name]
        # do os call in sub process
        subprocess.check_call(_c_args)
    else:
        pass

def _init_model():
    # get session
    _session_manager = SessionManager()
    if _session_manager is not None \
    and len(_session_manager.get_tables()) == 0:
        # get migrate params
        _db_url = Config().db_url()
        # get main repository path
        _repository = Config().db_repository()
        # migrate basic stuf
        migrate_main(argv=["version_control"], url=_db_url, repository=_repository)
        migrate_main(argv=["upgrade"], url=_db_url, repository=_repository)
        # 
        return True
    else:
        return False

def _init_data():
    # get session
    _session_manager = SessionManager(force=True)
    # little check
    if _session_manager is None:
        pass
    else:
        # init mapping after migrate
        model.init_mapping()
        # load default db value from yaml file
        _f = open("db_setup.yml")
        YamlDbLoader(_f).load()
        _f.close()

def _init_static_resources():
    # prepare statics from current resources
    _resource_path = Config().resource_path()
    _static_path = Config().static_path()
    # ensure resource paht
    if os.path.exists(_resource_path):
        pass
    else:
        os.makedirs(_resource_path)
    # remove old statics
    if os.path.exists(_static_path):
        shutil.rmtree(_static_path)
    else:
        pass
    # copy current user resources
    shutil.copytree(_resource_path, _static_path)

def _update_all_mo_files():
    # get query for language items
    _q = Query("Language")
    # get language codes
    _l_codes = _q.select(column_list=[_q.obj.language_code])
    # update mo file for each language code
    for _l_c in map(lambda _i: _i[0], _l_codes):
        I18nUpdater(_l_c).update_mo_file()

def _init_app(port_offset=0, parent=None, secure=False, use_ssl=True):
    # ensure valid page required for the portal
    # should have something like:
    # Page(realm=None)
    # if page is None, index url will be use as default
    # realm parameter will be added after by the twisted guard procedure
    _resource_cls = partial(Page, None)
    #
    if secure is True:
        # secure site
        _s_port = Config().get("secure_server>port", type_=int) + port_offset
        # start guard app
        if use_ssl is True:
            # get ssl context
            _ssl_context = SSLContext()
            # make secure app
            return application.make_secure_app(_s_port, _resource_cls,
                    parent, _ssl_context)
        else:
            # make simple app
            return application.make_simple_app(_s_port, _resource_cls,
                    parent, use_guard=True)
    else:
        # simple site
        _p_port = Config().get("public_server>port", type_=int) + port_offset
        # start public app
        return application.make_simple_app(_p_port, _resource_cls, parent)

def start_app(config=None, port_offset=0, parent=None, secure=False,
        use_ssl=False):
    """Application main entry point. Create a new twisted application according
    the given parameters. Default values are ok to start the application, but
    you need a minimal configuration file in your current directory.

    If no database found, will try to create a new database according the model
    found in the specified or default db repository. Then default from the
    `db_setup.yml` of the current folder will be set. Take a look to the
    `db_setup.yml-sample` file from the `bewype.foo.bar` project for a minimal
    setup.

    The `port_offset` option is little tricks to manage application proxying.
    The value will be added to the config server port,
    ex: port = 8080, if port_offset = 1 -> used port will be 8081

    :param config: SafeConfigParser object for the application
    :param port_offset: port offset value for proxying management, added to the
                        config server port.
    :param parent: parent aplication for the new twisted application
    :param secure: If set to True the application will use the `secure_server`
                   section of the configuration file. Then authentication can be
                   managed using the twisted guarded sessions.
    :param use_ssl: ssl listening flag, return an application using a SSLContext
                    is set to True. In that case cerfile and keyfile should
                    exist and correclty set in the configuration file.
    :return: the passed application updated with the new one or a new twisted
             application object.
    :rtype: `twisted.application.service.Application`
    """
    # init config manager singleton
    Config(config=config)
    # init logging
    _init_logging(port_offset, secure)
    # init db
    _check_db()
    # then migrate db
    if _init_model() is True:
        # model just inited -> add data now!
        _init_data()
    else:
        # do init mapping
        model.init_mapping()
    # init static resources
    _init_static_resources()
    # regen mo files
    _update_all_mo_files()
    # return app
    return _init_app(port_offset=port_offset, parent=parent,
            secure=secure, use_ssl=use_ssl)

