#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from datetime import datetime

# sqlalchemy import
from sqlalchemy import Column, ForeignKey, MetaData, Table, UniqueConstraint
from sqlalchemy.types import Boolean, DateTime, Integer, Unicode

# migrate import
from migrate import migrate_engine
metadata = MetaData(migrate_engine)

#### DUMMY TABLE

# orm definition for tests
bewype_dummy_table = Table(
        "bewype_dummy",
        metadata,
        Column("dummy_id", Integer, autoincrement=True, primary_key=True),
        Column("dummy_name", Unicode(32), unique=True, nullable=False),
        Column("dummy_str", Unicode(128)),
        Column("dummy_text", Unicode(512)),
        Column("dummy_color", Unicode(32)),
        Column("dummy_filename", Unicode(128)),
        Column("dummy_date", DateTime, default=datetime.now),
        Column("dummy_position", Integer),
        Column("parent_id", Integer,
            ForeignKey("bewype_dummy.dummy_id"))
        )

#### LANGUAGE TABLES

bewype_language_table = Table(
        "bewype_language",
        metadata,
        Column("language_id", Integer,
            autoincrement=True, primary_key=True),
        Column("language_name", Unicode(128),
            unique=True, nullable=False),
        Column("language_code", Unicode(4), unique=True),
        Column("display_name", Unicode(128), unique=True),
        )

bewype_country_table = Table(
        "bewype_country",
        metadata,
        Column("country_id", Integer,
            autoincrement=True, primary_key=True),
        Column("country_name", Unicode(128),
            unique=True, nullable=False),
        Column("country_code", Unicode(4), unique=True),
        Column("display_name", Unicode(128), unique=True),
        Column("language_id", Integer,
            ForeignKey("bewype_language.language_id"),
            nullable=False)
        )

#### PERMISSION TABLES

# This is the association table for the many-to-many relationship between
# groups and permissions.
group_permission_table = Table(
        "bewype_group_permission",
        metadata,
        Column("group_permission_id", Integer,
            autoincrement=True, primary_key=True),
        Column(
            "group_id", Integer,
            ForeignKey("bewype_group.group_id",
                onupdate="CASCADE", ondelete="CASCADE")
            ),
        Column(
            "permission_id", Integer,
            ForeignKey("bewype_permission.permission_id",
                onupdate="CASCADE", ondelete="CASCADE")
            )
        )

# This is the association table for the many-to-many relationship between
# groups and members - this is, the memberships.
user_group_table = Table(
        "bewype_user_group",
        metadata,
        Column("user_group_id", Integer,
            autoincrement=True, primary_key=True),
        Column(
            "user_id", Integer,
            ForeignKey("bewype_user.user_id",
                onupdate="CASCADE", ondelete="CASCADE")
            ),
        Column(
            "group_id", Integer,
            ForeignKey("bewype_group.group_id",
                onupdate="CASCADE", ondelete="CASCADE")
            )
        )

bewype_user_table = Table(
        "bewype_user",
        metadata,
        Column("user_id", Integer,
            autoincrement=True, primary_key=True),
        Column("user_name", Unicode(128),
            unique=True, nullable=False),
        Column("email_address", Unicode(128),
            unique=True, nullable=False),
        Column("display_name", Unicode(128),
            unique=True, nullable=False),
        Column("password", Unicode(80), key="_password",
            unique=True, nullable=False),
        Column("created", DateTime),
        Column("last_attempt", DateTime),
        Column("nb_of_attempt", Integer),
        Column("language_id", Integer,
            ForeignKey("bewype_language.language_id"))
        )

bewype_group_table = Table(
        "bewype_group",
        metadata,
        Column("group_id", Integer,
            autoincrement=True, primary_key=True),
        Column("group_name", Unicode(128),
            unique=True, nullable=False),
        Column("display_name", Unicode(128), unique=True),
        Column("description", Unicode(256))
        )

bewype_permission_table = Table(
        "bewype_permission",
        metadata,
        Column("permission_id", Integer,
            autoincrement=True, primary_key=True),
        Column("permission_name", Unicode(128),
            unique=True, nullable=False),
        Column("display_name", Unicode(128), unique=True),
        Column("description", Unicode(256))
        )

#### LAYOUT TABLES

bewype_layout_type_table = Table(
        "bewype_layout_type",
        metadata,
        Column("layout_type_id", Integer,
            autoincrement=True, primary_key=True),
        Column("layout_type_name", Unicode(128),
            unique=True, nullable=False),
        Column("display_name", Unicode(128), unique=True),
        Column("description", Unicode(256))
        )

bewype_layout_table = Table(
        "bewype_layout",
        metadata,
        Column("layout_id", Integer,
            autoincrement=True, primary_key=True),
        Column("display_name", Unicode(128), unique=True),
        Column("layout_type_id", Integer,
            ForeignKey("bewype_layout_type.layout_type_id"),
            nullable=False),
        Column("parent_id", Integer,
            ForeignKey("bewype_layout.layout_id")),
        Column("layout_position", Integer)
        )

bewype_page_table = Table(
        "bewype_page",
        metadata,
        Column("page_id", Integer,
            autoincrement=True, primary_key=True),
        Column("page_url", Unicode(128), unique=True),
        Column("layout_id", Integer,
            ForeignKey("bewype_layout.layout_id"))
        )

bewype_page_table = Table(
        "bewype_page_title",
        metadata,
        Column("page_title_id", Integer,
            autoincrement=True, primary_key=True),
        Column("display_name", Unicode(128), unique=True),
        Column("description", Unicode(256)),
        Column("language_id", Integer,
            ForeignKey("bewype_language.language_id")),
        Column("page_id", Integer,
            ForeignKey("bewype_page.page_id"))
        )

#### WIDGET TABLES

bewype_widget_table = Table(
        "bewype_widget",
        metadata,
        Column("widget_id", Integer,
            autoincrement=True, primary_key=True),
        Column("widget_name", Unicode(128),
            unique=True, nullable=False),
        Column("display_name", Unicode(128), unique=True),
        Column("description", Unicode(256))
        )

bewype_widget_element_table = Table(
        "bewype_widget_element",
        metadata,
        Column("widget_element_id", Integer,
            autoincrement=True, primary_key=True),
        Column("display_name", Unicode(128), nullable=False),
        Column("element_position", Integer, nullable=False),
        Column("layout_id", Integer,
            ForeignKey("bewype_layout.layout_id")),
        Column("widget_id", Integer,
            ForeignKey("bewype_widget.widget_id"),
            nullable=False),
        UniqueConstraint("element_position", "layout_id", "widget_id")
        )

###

widget_permission_table = Table(
        'bewype_widget_element_permission',
        metadata,
        Column("widget_element_permission_id", Integer,
            autoincrement=True, primary_key=True),
        Column('widget_element_id', Integer,
            ForeignKey('bewype_widget_element.widget_element_id')),
        Column('permission_id', Integer,
            ForeignKey('bewype_permission.permission_id'))
        )
#### CONFIG TABLES

bewype_config_table = Table(
        "bewype_config",
        metadata,
        Column("config_id", Integer,
            autoincrement=True, primary_key=True),
        Column("config_name", Unicode(128), nullable=False),
        Column("config_section", Unicode(128), nullable=False),
        Column("config_key", Unicode(128), nullable=False),
        Column("value_type", Unicode(128), nullable=False),
        Column("value_text", Unicode(512)),
        Column("language_id", Integer,
            ForeignKey("bewype_language.language_id"), nullable=False),
        Column("widget_element_id", Integer,
            ForeignKey("bewype_widget_element.widget_element_id"),
            nullable=False),
        UniqueConstraint("config_section", "config_key",
            "language_id", "widget_element_id")
        )

#### RESOURCE TABLES

bewype_resource_type_table = Table(
        "bewype_resource_type",
        metadata,
        Column("resource_type_id", Integer,
            autoincrement=True, primary_key=True),
        Column("resource_type_name", Unicode(128),
            unique=True, nullable=False),
        Column("display_name", Unicode(128),
            unique=True, nullable=False),
        Column("description", Unicode(256))
        )

bewype_resource_table = Table(
        "bewype_resource",
        metadata,
        Column("resource_id", Integer,
            autoincrement=True, primary_key=True),
        Column("default", Boolean),
        Column("display_name", Unicode(128), nullable=False),
        Column("file_name", Unicode(128), nullable=False),
        Column("language_id", Integer,
            ForeignKey("bewype_language.language_id"),
            nullable=False),
        Column("resource_type_id", Integer,
            ForeignKey("bewype_resource_type.resource_type_id"),
            nullable=False),
        Column("widget_element_id", Integer,
            ForeignKey("bewype_widget_element.widget_element_id"),
            nullable=False),
        UniqueConstraint("display_name", "language_id", "widget_element_id"),
        UniqueConstraint("file_name", "language_id", "widget_element_id")
        )

#### MENU TABLES

bewype_menu_table = Table(
        "bewype_menu",
        metadata,
        Column("menu_id", Integer,
            autoincrement=True, primary_key=True),
        Column("display_name", Unicode(128),
            nullable=False),
        Column("description", Unicode(256)),
        Column("language_id", Integer,
            ForeignKey("bewype_language.language_id"),
            nullable=False),
        Column("widget_element_id", Integer,
            ForeignKey("bewype_widget_element.widget_element_id")),
        )

bewype_menu_element_table = Table(
        "bewype_menu_element",
        metadata,
        Column("menu_element_id", Integer,
            autoincrement=True, primary_key=True),
        Column("display_name", Unicode(128),
            nullable=False),
        Column("description", Unicode(256)),
        Column("link_to_page", Boolean),
        Column("menu_element_position", Integer),
        Column("language_id", Integer,
            ForeignKey("bewype_language.language_id"),
            nullable=False),
        Column("menu_id", Integer,
            ForeignKey("bewype_menu.menu_id")),
        Column("parent_id", Integer,
            ForeignKey("bewype_menu_element.menu_element_id")),
        Column("resource_id", Integer,
            ForeignKey("bewype_resource.resource_id")),
        Column("widget_element_id", Integer,
            ForeignKey("bewype_widget_element.widget_element_id"),
            nullable=False),
        )

def upgrade():
    # Upgrade operations go here. Don't create your own engine; use the engine
    # named 'migrate_engine' imported from migrate.
    #
    metadata.create_all(migrate_engine)

def downgrade():
    # Operations to reverse the above upgrade go here.
    #
    metadata.drop_all(migrate_engine)

